@@ -93,33 +93,30 @@
 //! copied and the writer will emit them accordingly.
 //!
 //! Namespaces need to be registered or the XML generated will be malformed.
-extern crate xml;
-extern crate string_cache;
-
-use std::cmp::Ord;
-use std::collections::BTreeMap;
-use std::collections::btree_map::Iter as BTreeMapIter;
-use std::io::{Read, Write};
-use std::io;
-use std::fmt;
-use std::mem;
-use std::str::Utf8Error;
-use std::ops::Deref;
-use std::cmp::Ordering;
-use std::hash::{Hash, Hasher};
 use std::borrow::Cow;
+use std::cmp::Ord;
+use std::cmp::Ordering;
+use std::collections::btree_map::Iter as BTreeMapIter;
+use std::collections::BTreeMap;
+use std::fmt;
+use std::hash::{Hash, Hasher};
+use std::io;
+use std::io::{Read, Write};
+use std::mem;
+use std::ops::Deref;
 use std::rc::Rc;
+use std::str::Utf8Error;
 
 use string_cache::DefaultAtom as Atom;
 
-use xml::reader::{EventReader, ParserConfig, XmlEvent, Error as XmlReadError,
-                  ErrorKind as XmlReadErrorKind};
-use xml::writer::{EventWriter, XmlEvent as XmlWriteEvent, Error as XmlWriteError};
-use xml::common::{XmlVersion, Position as XmlPosition};
 use xml::attribute::{Attribute, OwnedAttribute};
+use xml::common::{Position as XmlPosition, XmlVersion};
 use xml::name::{Name, OwnedName};
-use xml::namespace::{Namespace as XmlNamespaceMap, NS_EMPTY_URI,
-                     NS_XMLNS_URI, NS_XML_URI};
+use xml::namespace::{Namespace as XmlNamespaceMap, NS_EMPTY_URI, NS_XMLNS_URI, NS_XML_URI};
+use xml::reader::{
+    Error as XmlReadError, ErrorKind as XmlReadErrorKind, EventReader, ParserConfig, XmlEvent,
+};
+use xml::writer::{Error as XmlWriteError, EventWriter, XmlEvent as XmlWriteEvent};
 use xml::EmitterConfig;
 
 enum XmlAtom<'a> {
@@ -142,7 +139,7 @@ impl<'a> Deref for XmlAtom<'a> {
 impl<'a> XmlAtom<'a> {
     #[inline(always)]
     pub fn borrow(&self) -> &str {
-        &self
+        self
     }
 }
 
@@ -258,8 +255,8 @@ impl<'a> QName<'a> {
         }
 
         QName {
-            ns: ns,
-            name: name.unwrap_or_else(|| XmlAtom::Borrowed(s)),
+            ns,
+            name: name.unwrap_or(XmlAtom::Borrowed(s)),
         }
     }
 
@@ -276,7 +273,7 @@ impl<'a> QName<'a> {
     /// Creates a qualified name from a namespace and name.
     pub fn from_ns_name(ns: Option<&'a str>, name: &'a str) -> QName<'a> {
         QName {
-            ns: ns.map(|x| XmlAtom::Borrowed(x)),
+            ns: ns.map(XmlAtom::Borrowed),
             name: XmlAtom::Borrowed(name),
         }
     }
@@ -300,7 +297,10 @@ impl<'a> QName<'a> {
     pub fn share(&self) -> QName<'static> {
         QName {
             name: XmlAtom::Shared(Atom::from(self.name.borrow())),
-            ns: self.ns.as_ref().map(|x| XmlAtom::Shared(Atom::from(x.borrow()))),
+            ns: self
+                .ns
+                .as_ref()
+                .map(|x| XmlAtom::Shared(Atom::from(x.borrow()))),
         }
     }
 
@@ -309,21 +309,21 @@ impl<'a> QName<'a> {
             name: XmlAtom::Shared(Atom::from(name.local_name)),
             ns: match name.namespace {
                 Some(ns) => {
-                    if ns.len() > 0 {
+                    if !ns.is_empty() {
                         Some(XmlAtom::Shared(Atom::from(ns)))
                     } else {
                         None
                     }
-                },
+                }
                 _ => None,
-            }
+            },
         }
     }
 }
 
 impl<'a> PartialEq for QName<'a> {
     fn eq(&self, other: &QName<'a>) -> bool {
-         self.name() == other.name() && self.ns() == other.ns()
+        self.name() == other.name() && self.ns() == other.ns()
     }
 }
 
@@ -407,7 +407,10 @@ impl NamespaceMap {
         let mut i = 1;
         loop {
             let random_prefix = format!("ns{}", i);
-            if !self.prefix_to_ns.contains_key(&XmlAtom::Borrowed(&random_prefix)) {
+            if !self
+                .prefix_to_ns
+                .contains_key(&XmlAtom::Borrowed(&random_prefix))
+            {
                 return XmlAtom::Shared(Atom::from(random_prefix));
             }
             i += 1;
@@ -476,7 +479,7 @@ pub struct Element {
     nsmap: Option<Rc<NamespaceMap>>,
     emit_nsmap: bool,
     text: Option<String>,
-    tail: Option<String>
+    tail: Option<String>,
 }
 
 /// An iterator over children of an element.
@@ -511,19 +514,16 @@ pub struct FindChildrenMut<'a> {
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Ord, PartialOrd)]
 pub struct Position {
     line: u64,
-    column: u64
+    column: u64,
 }
 
 impl Position {
     /// Creates a new position.
     pub fn new(line: u64, column: u64) -> Position {
-        Position {
-            line: line,
-            column: column,
-        }
+        Position { line, column }
     }
 
-    fn from_xml_position(pos: &XmlPosition) -> Position {
+    fn from_xml_position(pos: &dyn XmlPosition) -> Position {
         let pos = pos.position();
         Position::new(pos.row, pos.column)
     }
@@ -550,7 +550,7 @@ pub enum Error {
     /// The XML is invalid
     MalformedXml {
         msg: Cow<'static, str>,
-        pos: Position
+        pos: Position,
     },
     /// An IO Error
     Io(io::Error),
@@ -560,7 +560,7 @@ pub enum Error {
     /// example, the XML contains processing instructions.
     UnexpectedEvent {
         msg: Cow<'static, str>,
-        pos: Position
+        pos: Position,
     },
     /// A namespace prefix was already used
     DuplicateNamespacePrefix,
@@ -570,8 +570,8 @@ impl Error {
     /// Returns the position of the error if known
     pub fn position(&self) -> Option<Position> {
         match self {
-            &Error::MalformedXml { pos, .. } => Some(pos),
-            &Error::UnexpectedEvent { pos, .. } => Some(pos),
+            Error::MalformedXml { pos, .. } => Some(*pos),
+            Error::UnexpectedEvent { pos, .. } => Some(*pos),
             _ => None,
         }
     }
@@ -589,34 +589,25 @@ impl Error {
 
 impl fmt::Display for Error {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        match self {
-            &Error::MalformedXml { ref pos, ref msg } =>
-                write!(f, "Malformed XML: {} ({})", msg, pos),
-            &Error::Io(ref e) => write!(f, "{}", e),
-            &Error::Utf8(ref e) => write!(f, "{}", e),
-            &Error::UnexpectedEvent { ref msg, .. } =>
-                write!(f, "Unexpected XML event: {}", msg),
-            &Error::DuplicateNamespacePrefix => write!(
-                f, "Encountered duplicated namespace prefix"),
+        match *self {
+            Error::MalformedXml { ref pos, ref msg } => {
+                write!(f, "Malformed XML: {} ({})", msg, pos)
+            }
+            Error::Io(ref e) => write!(f, "{}", e),
+            Error::Utf8(ref e) => write!(f, "{}", e),
+            Error::UnexpectedEvent { ref msg, .. } => write!(f, "Unexpected XML event: {}", msg),
+            Error::DuplicateNamespacePrefix => {
+                write!(f, "Encountered duplicated namespace prefix")
+            }
         }
     }
 }
 
 impl std::error::Error for Error {
-    fn description(&self) -> &str {
-        match self {
-            &Error::MalformedXml { .. } => "Malformed XML",
-            &Error::Io(..) => "IO error",
-            &Error::Utf8(..) => "utf-8 error",
-            &Error::UnexpectedEvent { .. } => "Unexpected XML element",
-            &Error::DuplicateNamespacePrefix => "Duplicated namespace prefix",
-        }
-    }
-
-    fn cause(&self) -> Option<&std::error::Error> {
-        match self {
-            &Error::Io(ref e) => Some(e),
-            &Error::Utf8(ref e) => Some(e),
+    fn cause(&self) -> Option<&dyn std::error::Error> {
+        match *self {
+            Error::Io(ref e) => Some(e),
+            Error::Utf8(ref e) => Some(e),
             _ => None,
         }
     }
@@ -624,20 +615,17 @@ impl std::error::Error for Error {
 
 impl From<XmlReadError> for Error {
     fn from(err: XmlReadError) -> Error {
-        use std::error::Error as StdError;
-        match err.kind() {
-            &XmlReadErrorKind::Io(ref err) => Error::Io(
-                io::Error::new(err.kind(), err.description())),
-            &XmlReadErrorKind::Utf8(ref err) => Error::Utf8(err.clone()),
-            &XmlReadErrorKind::UnexpectedEof => Error::Io(
-                io::Error::new(io::ErrorKind::UnexpectedEof,
-                               "Encountered unexpected eof")),
-            &XmlReadErrorKind::Syntax(ref msg) => {
-                Error::MalformedXml {
-                    msg: msg.clone(),
-                    pos: Position::from_xml_position(&err),
-                }
-            }
+        match *err.kind() {
+            XmlReadErrorKind::Io(ref err) => Error::Io(io::Error::new(err.kind(), err.to_string())),
+            XmlReadErrorKind::Utf8(err) => Error::Utf8(err),
+            XmlReadErrorKind::UnexpectedEof => Error::Io(io::Error::new(
+                io::ErrorKind::UnexpectedEof,
+                "Encountered unexpected eof",
+            )),
+            XmlReadErrorKind::Syntax(ref msg) => Error::MalformedXml {
+                msg: msg.clone(),
+                pos: Position::from_xml_position(&err),
+            },
         }
     }
 }
@@ -646,7 +634,7 @@ impl From<XmlWriteError> for Error {
     fn from(err: XmlWriteError) -> Error {
         match err {
             XmlWriteError::Io(err) => Error::Io(err),
-            err => { return Err(err).unwrap(); }
+            err => Err(err).unwrap(),
         }
     }
 }
@@ -707,7 +695,7 @@ impl<'a> Iterator for FindChildrenMut<'a> {
 
     fn next(&mut self) -> Option<&'a mut Element> {
         use std::borrow::Borrow;
-        let tag: &QName = &self.tag.borrow();
+        let tag: &QName = self.tag.borrow();
         self.child_iter.find(|x| x.tag() == tag)
     }
 }
@@ -733,11 +721,11 @@ impl Element {
         Element::new_with_nsmap(&tag.as_qname(), reference.nsmap.clone())
     }
 
-    fn new_with_nsmap<'a>(tag: &QName<'a>, nsmap: Option<Rc<NamespaceMap>>) -> Element {
+    fn new_with_nsmap(tag: &QName<'_>, nsmap: Option<Rc<NamespaceMap>>) -> Element {
         let mut rv = Element {
             tag: tag.share(),
             attributes: BTreeMap::new(),
-            nsmap: nsmap,
+            nsmap,
             emit_nsmap: false,
             children: vec![],
             text: None,
@@ -752,25 +740,35 @@ impl Element {
 
     /// Parses some XML data into an `Element` from a reader.
     pub fn from_reader<R: Read>(r: R) -> Result<Element, Error> {
-        let cfg = ParserConfig::new()
-            .whitespace_to_characters(true);
+        let cfg = ParserConfig::new().whitespace_to_characters(true);
         let mut reader = cfg.create_reader(r);
         loop {
             match reader.next() {
-                Ok(XmlEvent::StartElement { name, attributes, namespace }) => {
+                Ok(XmlEvent::StartElement {
+                    name,
+                    attributes,
+                    namespace,
+                }) => {
                     return Element::from_start_element(
-                        name, attributes, namespace, None, &mut reader);
+                        name,
+                        attributes,
+                        namespace,
+                        None,
+                        &mut reader,
+                    );
                 }
-                Ok(XmlEvent::Comment(..)) |
-                Ok(XmlEvent::Whitespace(..)) |
-                Ok(XmlEvent::StartDocument { .. }) |
-                Ok(XmlEvent::ProcessingInstruction { .. }) => {
+                Ok(XmlEvent::Comment(..))
+                | Ok(XmlEvent::Whitespace(..))
+                | Ok(XmlEvent::StartDocument { .. })
+                | Ok(XmlEvent::ProcessingInstruction { .. }) => {
                     continue;
                 }
-                Ok(_) => return Err(Error::UnexpectedEvent {
-                    msg: Cow::Borrowed("xml construct"),
-                    pos: Position::from_xml_position(&reader),
-                }),
+                Ok(_) => {
+                    return Err(Error::UnexpectedEvent {
+                        msg: Cow::Borrowed("xml construct"),
+                        pos: Position::from_xml_position(&reader),
+                    })
+                }
                 Err(e) => return Err(e.into()),
             }
         }
@@ -802,7 +800,11 @@ impl Element {
     /// elements.  The reason for this is that there is no way to ignore
     /// this information automatically in the absence of DTD support which
     /// is not really planned.
-    pub fn to_writer_with_options<W: Write>(&self, w: W, options: WriteOptions) -> Result<(), Error> {
+    pub fn to_writer_with_options<W: Write>(
+        &self,
+        w: W,
+        options: WriteOptions,
+    ) -> Result<(), Error> {
         let mut writer = EmitterConfig::new()
             .write_document_declaration(options.xml_prolog.is_some())
             .create_writer(w);
@@ -862,7 +864,7 @@ impl Element {
         }
 
         w.write(XmlWriteEvent::StartElement {
-            name: name,
+            name,
             attributes: Cow::Owned(attributes),
             namespace: Cow::Owned(namespace),
         })?;
@@ -880,20 +882,18 @@ impl Element {
             }
         }
 
-        w.write(XmlWriteEvent::EndElement {
-            name: Some(name),
-        })?;
+        w.write(XmlWriteEvent::EndElement { name: Some(name) })?;
 
         Ok(())
     }
 
-    fn from_start_element<R: Read>(name: OwnedName,
-                                   attributes: Vec<OwnedAttribute>,
-                                   namespace: XmlNamespaceMap,
-                                   parent_nsmap: Option<Rc<NamespaceMap>>,
-                                   reader: &mut EventReader<R>)
-        -> Result<Element, Error>
-    {
+    fn from_start_element<R: Read>(
+        name: OwnedName,
+        attributes: Vec<OwnedAttribute>,
+        namespace: XmlNamespaceMap,
+        parent_nsmap: Option<Rc<NamespaceMap>>,
+        reader: &mut EventReader<R>,
+    ) -> Result<Element, Error> {
         let mut root = Element {
             tag: QName::from_owned_name(name),
             attributes: BTreeMap::new(),
@@ -904,7 +904,8 @@ impl Element {
             tail: None,
         };
         for attr in attributes {
-            root.attributes.insert(QName::from_owned_name(attr.name), attr.value);
+            root.attributes
+                .insert(QName::from_owned_name(attr.name), attr.value);
         }
 
         if !namespace.is_essentially_empty() {
@@ -917,15 +918,13 @@ impl Element {
         Ok(root)
     }
 
-    fn parse_children<R: Read>(&mut self,
-                               reader: &mut EventReader<R>)
-        -> Result<(), Error>
-    {
+    fn parse_children<R: Read>(&mut self, reader: &mut EventReader<R>) -> Result<(), Error> {
         loop {
             match reader.next() {
                 Ok(XmlEvent::EndElement { ref name }) => {
-                    if &name.local_name == self.tag.name() &&
-                       name.namespace.as_ref().map(|x| x.as_str()) == self.tag.ns() {
+                    if name.local_name == self.tag.name()
+                        && name.namespace.as_deref() == self.tag.ns()
+                    {
                         return Ok(());
                     } else {
                         return Err(Error::UnexpectedEvent {
@@ -934,9 +933,18 @@ impl Element {
                         });
                     }
                 }
-                Ok(XmlEvent::StartElement { name, attributes, namespace }) => {
+                Ok(XmlEvent::StartElement {
+                    name,
+                    attributes,
+                    namespace,
+                }) => {
                     self.children.push(Element::from_start_element(
-                        name, attributes, namespace, self.nsmap.clone(), reader)?);
+                        name,
+                        attributes,
+                        namespace,
+                        self.nsmap.clone(),
+                        reader,
+                    )?);
                 }
                 Ok(XmlEvent::Characters(s)) => {
                     let child_count = self.children.len();
@@ -949,10 +957,10 @@ impl Element {
                 Ok(XmlEvent::CData(s)) => {
                     self.text = Some(s);
                 }
-                Ok(XmlEvent::Comment(..)) |
-                Ok(XmlEvent::Whitespace(..)) |
-                Ok(XmlEvent::StartDocument { .. }) |
-                Ok(XmlEvent::ProcessingInstruction { .. }) => {
+                Ok(XmlEvent::Comment(..))
+                | Ok(XmlEvent::Whitespace(..))
+                | Ok(XmlEvent::StartDocument { .. })
+                | Ok(XmlEvent::ProcessingInstruction { .. }) => {
                     continue;
                 }
                 Ok(_) => {
@@ -973,7 +981,7 @@ impl Element {
     /// Note that this does not trim or modify whitespace so the return
     /// value might contain structural information from the XML file.
     pub fn text(&self) -> &str {
-        self.text.as_ref().map(|x| x.as_str()).unwrap_or("")
+        self.text.as_deref().unwrap_or("")
     }
 
     /// Sets a new text value for the tag.
@@ -991,7 +999,7 @@ impl Element {
     ///
     /// The tail is the text following an element.
     pub fn tail(&self) -> &str {
-        self.tail.as_ref().map(|x| x.as_str()).unwrap_or("")
+        self.tail.as_deref().unwrap_or("")
     }
 
     /// Sets a new tail text value for the tag.
@@ -1078,7 +1086,7 @@ impl Element {
     }
 
     /// Returns an iterator over all children.
-    pub fn children<'a>(&'a self) -> Children<'a> {
+    pub fn children(&self) -> Children<'_> {
         Children {
             idx: 0,
             element: self,
@@ -1086,15 +1094,14 @@ impl Element {
     }
 
     /// Returns a mutable iterator over all children.
-    pub fn children_mut<'a>(&'a mut self) -> ChildrenMut<'a> {
+    pub fn children_mut(&mut self) -> ChildrenMut<'_> {
         ChildrenMut {
             iter: self.children.iter_mut(),
         }
     }
 
     /// Returns all children with the given name.
-    pub fn find_all<'a, Q: AsQName<'a>>(&'a self, tag: Q) -> FindChildren<'a>
-    {
+    pub fn find_all<'a, Q: AsQName<'a>>(&'a self, tag: Q) -> FindChildren<'a> {
         FindChildren {
             tag: tag.as_qname(),
             child_iter: self.children(),
@@ -1102,8 +1109,7 @@ impl Element {
     }
 
     /// Returns all children with the given name.
-    pub fn find_all_mut<'a, Q: AsQName<'a>>(&'a mut self, tag: Q) -> FindChildrenMut<'a>
-    {
+    pub fn find_all_mut<'a, Q: AsQName<'a>>(&'a mut self, tag: Q) -> FindChildrenMut<'a> {
         FindChildrenMut {
             tag: tag.as_qname(),
             child_iter: self.children_mut(),
@@ -1137,9 +1143,12 @@ impl Element {
     ///
     /// This returns a reference to the element so you can chain the calls.
     pub fn set_attr<'a, Q: AsQName<'a>, S: Into<String>>(
-        &'a mut self, name: Q, value: S) -> &'a mut Element
-    {
-        self.attributes.insert(name.as_qname().share(), value.into());
+        &'a mut self,
+        name: Q,
+        value: S,
+    ) -> &'a mut Element {
+        self.attributes
+            .insert(name.as_qname().share(), value.into());
         self
     }
 
@@ -1163,7 +1172,7 @@ impl Element {
     }
 
     /// Returns an iterator over all attributes
-    pub fn attrs<'a>(&'a self) -> Attrs<'a> {
+    pub fn attrs(&self) -> Attrs<'_> {
         Attrs {
             iter: self.attributes.iter(),
         }
@@ -1194,10 +1203,10 @@ impl Element {
     /// This optionally also registers a specific prefix however if that prefix
     /// is already used a random one is used instead.
     pub fn register_namespace(&mut self, url: &str, prefix: Option<&str>) {
-        if self.get_namespace_prefix(url).is_none() {
-            if self.get_nsmap_mut().register_if_missing(url, prefix) {
-                self.emit_nsmap = true;
-            }
+        if self.get_namespace_prefix(url).is_none()
+            && self.get_nsmap_mut().register_if_missing(url, prefix)
+        {
+            self.emit_nsmap = true;
         }
     }
 
@@ -1259,13 +1268,13 @@ pub enum XmlProlog {
 
 /// A struct that define write options.
 pub struct WriteOptions {
-    xml_prolog: Option<XmlProlog>
+    xml_prolog: Option<XmlProlog>,
 }
 
 impl Default for WriteOptions {
     fn default() -> WriteOptions {
         WriteOptions {
-            xml_prolog: Some(XmlProlog::Version10)
+            xml_prolog: Some(XmlProlog::Version10),
         }
     }
 }
